local LCA = LibCombatAlerts


--------------------------------------------------------------------------------
-- Constants
--------------------------------------------------------------------------------

local POLLING_INTERVAL = 100

local DEFAULT_OPTIONS = {
	color = 0xFFFFFFFF,
	size = 4,
}


--------------------------------------------------------------------------------
-- LCA_MoveableControl
--------------------------------------------------------------------------------

LCA_MoveableControl = ZO_Object:Subclass()
local LCA_MoveableControl = LCA_MoveableControl

local nextInstanceId = 1

local function Identifier( x )
	return "LCA_MoveableControl_" .. x
end

-- Preserve GuiRoot to guard against redefinition
local GuiRoot = GuiRoot

function LCA_MoveableControl:New( control, options )
	local obj = ZO_Object.New(self)

	obj.name = Identifier(nextInstanceId)
	nextInstanceId = nextInstanceId + 1

	obj.control = control
	control.lcamc = obj

	-- Initialize anchor markers

	local opt = LCA.PopulateOptions(options, DEFAULT_OPTIONS)

	obj.anchorH = control:GetNamedChild("AnchorH")
	obj.anchorH:SetEdgeColor(0, 0, 0, 0)
	obj.anchorH:SetCenterColor(LCA.UnpackRGBA(opt.color))
	obj.anchorH:SetHeight(opt.size)

	obj.anchorV = control:GetNamedChild("AnchorV")
	obj.anchorV:SetEdgeColor(0, 0, 0, 0)
	obj.anchorV:SetCenterColor(LCA.UnpackRGBA(opt.color))
	obj.anchorV:SetWidth(opt.size)

	return obj
end

function LCA_MoveableControl:SetCallback( fn )
	self.repositionCallback = fn
end

function LCA_MoveableControl:UpdateAnchorMarkers( )
	local anchorPointH, anchorPointV

	if (self.anchorPoint == TOPLEFT or self.anchorPoint == TOPRIGHT) then
		anchorPointH = TOPLEFT
	else
		anchorPointH = BOTTOMLEFT
	end

	if (self.anchorPoint == TOPLEFT or self.anchorPoint == BOTTOMLEFT) then
		anchorPointV = TOPLEFT
	else
		anchorPointV = TOPRIGHT
	end

	self.anchorH:ClearAnchors()
	self.anchorH:SetAnchor(anchorPointH)
	self.anchorH:SetAnchor(RIGHT, nil, nil, nil, nil, ANCHOR_CONSTRAINS_X)

	self.anchorV:ClearAnchors()
	self.anchorV:SetAnchor(anchorPointV)
	self.anchorV:SetAnchor(BOTTOM, nil, nil, nil, nil, ANCHOR_CONSTRAINS_Y)
end

function LCA_MoveableControl:UpdatePosition( pos )
	local dx, dy

	if (pos.top and pos.right) then
		self.anchorPoint = TOPRIGHT
		dx, dy = pos.right, pos.top
	elseif (pos.bottom and pos.left) then
		self.anchorPoint = BOTTOMLEFT
		dx, dy = pos.left, pos.bottom
	elseif (pos.bottom and pos.right) then
		self.anchorPoint = BOTTOMRIGHT
		dx, dy = pos.right, pos.bottom
	else
		self.anchorPoint = TOPLEFT
		dx, dy = pos.left or 0, pos.top or 0
	end

	self.control:ClearAnchors()
	self.control:SetAnchor(self.anchorPoint, GuiRoot, self.anchorPoint, dx, dy)
end

function LCA_MoveableControl:ToggleMovement( enable )
	if (enable) then
		EVENT_MANAGER:RegisterForUpdate(self.name, POLLING_INTERVAL, function() self:MovementPoll() end)
		self:MovementPoll()
		self:UpdateAnchorMarkers()
		self.anchorH:SetHidden(false)
		self.anchorV:SetHidden(false)
	else
		EVENT_MANAGER:UnregisterForUpdate(self.name)
		self:MovementPoll()
		self.anchorH:SetHidden(true)
		self.anchorV:SetHidden(true)
	end
end

function LCA_MoveableControl:MovementPoll( )
	if ((self.anchorPoint == TOPLEFT or self.anchorPoint == TOPRIGHT) and self.control:GetBottom() >= GuiRoot:GetHeight()) then
		self.anchorPoint = (self.anchorPoint == TOPLEFT) and BOTTOMLEFT or BOTTOMRIGHT
		self:UpdateAnchorMarkers()
	elseif ((self.anchorPoint == BOTTOMLEFT or self.anchorPoint == BOTTOMRIGHT) and self.control:GetTop() <= 0) then
		self.anchorPoint = (self.anchorPoint == BOTTOMLEFT) and TOPLEFT or TOPRIGHT
		self:UpdateAnchorMarkers()
	end

	if ((self.anchorPoint == TOPLEFT or self.anchorPoint == BOTTOMLEFT) and self.control:GetRight() >= GuiRoot:GetWidth()) then
		self.anchorPoint = (self.anchorPoint == TOPLEFT) and TOPRIGHT or BOTTOMRIGHT
		self:UpdateAnchorMarkers()
	elseif ((self.anchorPoint == TOPRIGHT or self.anchorPoint == BOTTOMRIGHT) and self.control:GetLeft() <= 0) then
		self.anchorPoint = (self.anchorPoint == TOPRIGHT) and TOPLEFT or BOTTOMLEFT
		self:UpdateAnchorMarkers()
	end
end

function LCA_MoveableControl:OnMoveStart( )
	self:ToggleMovement(true)
end

function LCA_MoveableControl:OnMoveStop( control )
	self:ToggleMovement(false)

	local pos = { }
	if (self.anchorPoint == TOPLEFT or self.anchorPoint == TOPRIGHT) then
		pos.top = control:GetTop()
	else
		pos.bottom = control:GetBottom() - GuiRoot:GetHeight()
	end
	if (self.anchorPoint == TOPLEFT or self.anchorPoint == BOTTOMLEFT) then
		pos.left = control:GetLeft()
	else
		pos.right = control:GetRight() - GuiRoot:GetWidth()
	end

	self:UpdatePosition(pos)

	if (type(self.repositionCallback) == "function") then
		self.repositionCallback(pos)
	end
end


--------------------------------------------------------------------------------
-- XML Handlers
--------------------------------------------------------------------------------

function LCA_MoveableControl_OnMoveStart( control )
	if (control.lcamc) then
		control.lcamc:OnMoveStart()
	end
end

function LCA_MoveableControl_OnMoveStop( control )
	if (control.lcamc) then
		control.lcamc:OnMoveStop(control)
	end
end
